package kz.project.printedFormsService.logging;

import jakarta.servlet.http.HttpServletRequest;
import kz.project.printedFormsService.converter.JwtAuthConverter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.format.datetime.DateFormatter;
import org.springframework.http.HttpHeaders;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.io.File;
import java.io.FileWriter;
import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Date;
import java.util.Locale;

/* Класс для логгирования старта и завершения процессов в формате SIEM и на естественном языке */
@Component
public class ProcessLogger {
    private static final Logger LOGGER = LoggerFactory.getLogger(ProcessLogger.class);
    private static final String DEFAULT_FILENAME = "siem-processes.log";
    private static final String DEFAULT_DELIMITER = " ";

    @Value("${process-logger.file-name}")
    private String logFileName;
    @Value("${process-logger.delimiter}")
    private String delimiter;
    @Value("${spring.application.name}")
    private String serviceName;
    @Value("${process-logger.start-prefix}")
    private String startPrefix;
    @Value("${process-logger.finish-prefix}")
    private String finishPrefix;
    @Value("${process-logger.error-prefix}")
    private String errorPrefix;

    private DateFormatter dateFormatter;
    private Locale locale;
    private ZoneId zoneId;

    public ProcessLogger(DateFormatter dateFormatter, Locale locale, ZoneId zoneId) {
        this.dateFormatter = dateFormatter;
        this.locale = locale;
        this.zoneId = zoneId;
    }

    public void start(Class javaClass, String processName, String description){
        write(javaClass, processName, startPrefix, description);
    }

    public void finish(Class javaClass, String processName, String description){
        write(javaClass, processName, finishPrefix, description);
    }

    public void error(Class javaClass, String processName, String description){
        write(javaClass, processName, errorPrefix, description);
    }

    public void write(Class javaClass, String processName, String descriptionPrefix, String description){
        try {
            HttpServletRequest request = getRequest();

            String delimiter = this.delimiter != null ? this.delimiter : DEFAULT_DELIMITER;
            String fileName = this.logFileName != null ? this.logFileName : DEFAULT_FILENAME;
            String ip = request.getRemoteAddr();
            String userName = null;

            String authHeader = request.getHeader(HttpHeaders.AUTHORIZATION);
            if (authHeader != null){
                userName = JwtAuthConverter.getUsername(authHeader.substring(7));
            } else {
                authHeader = request.getHeader("X-API-Key");
                if (authHeader != null) userName = authHeader.substring(9);
            }

            String logRow = String.format(
                    "%s%s%s.%s%s%s%s%s%s%s%s%s%s",
                    serviceName,
                    delimiter,
                    javaClass.getSimpleName(),
                    processName,
                    delimiter,
                    getTodayDateTimeString(),
                    delimiter,
                    userName != null ? userName : "system-request",
                    delimiter,
                    ip,
                    delimiter,
                    descriptionPrefix,
                    description
            );

            File file = new File(fileName);
            boolean isCreated = file.createNewFile();

            FileWriter writer = new FileWriter(fileName, true);
            writer.append(logRow).append(System.lineSeparator());
            writer.close();
        } catch (Exception e){
            LOGGER.error(e.getMessage(), e);
        }
    }

    private String getTodayDateTimeString(){
        Instant instant = new Date().toInstant();
        ZonedDateTime zonedDateTime = instant.atZone(this.zoneId);

        Date today = Date.from(zonedDateTime.toInstant());
        return dateFormatter.print(today, locale);
    }

    private HttpServletRequest getRequest(){
        return ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
    }
}
